import { Controller } from "@hotwired/stimulus"

export default class extends Controller {
  static targets = ["requiredFields", "customFields", "importJobClass", "customTemplate"]
  
  connect() {
    // Initialize with at least one custom field if none exist
    if (this.hasCustomFieldsTarget && this.customFieldsTarget.querySelectorAll('.custom-field-row').length === 0) {
      this.addCustomField()
    }
    
    // Load required fields based on the initial selection
    if (this.hasImportJobClassTarget && this.importJobClassTarget.value) {
      this.importJobClassChanged()
    }
  }
  
  importJobClassChanged() {
    if (!this.hasImportJobClassTarget || !this.hasRequiredFieldsTarget) return
    
    const selectedClass = this.importJobClassTarget.value
    if (!selectedClass) {
      this.requiredFieldsTarget.innerHTML = '<p class="text-sm text-gray-500 dark:text-gray-400">Select an import job class to see required fields</p>'
      return
    }
    
    // Store the current data values
    const currentData = this.getCurrentDataValues()
    
    // Fetch the required fields for the selected import job class
    fetch(`/conferences/required_fields?import_job_class=${encodeURIComponent(selectedClass)}`)
      .then(response => response.json())
      .then(data => {
        // Clear the required fields container
        this.requiredFieldsTarget.innerHTML = ''
        
        if (data.required_fields.length === 0) {
          this.requiredFieldsTarget.innerHTML = '<p class="text-sm text-gray-500 dark:text-gray-400">No required fields for this import job class</p>'
          return
        }
        
        // Add the required fields
        data.required_fields.forEach(field => {
          const metadata = data.metadata[field] || {}
          const fieldValue = currentData[field] || ''
          
          const fieldHtml = this.generateRequiredFieldHtml(
            field,
            metadata.title || this.humanize(field),
            metadata.description || '',
            metadata.placeholder || '',
            fieldValue,
            metadata.required || false
          )
          
          this.requiredFieldsTarget.insertAdjacentHTML('beforeend', fieldHtml)
          
          // Remove this field from currentData as it's now a required field
          delete currentData[field]
        })
        
        // Convert any remaining data fields to custom fields
        this.convertToCustomFields(currentData)
      })
      .catch(error => {
        console.error('Error fetching required fields:', error)
        this.requiredFieldsTarget.innerHTML = '<p class="text-sm text-red-500">Error loading required fields</p>'
      })
  }
  
  getCurrentDataValues() {
    const dataValues = {}
    
    // Get values from required fields
    if (this.hasRequiredFieldsTarget) {
      const inputs = this.requiredFieldsTarget.querySelectorAll('input[name^="data["]')
      inputs.forEach(input => {
        const matches = input.name.match(/data\[(.*?)\]/)
        if (matches && matches.length > 1) {
          dataValues[matches[1]] = input.value
        }
      })
    }
    
    // Get values from custom fields
    if (this.hasCustomFieldsTarget) {
      const rows = this.customFieldsTarget.querySelectorAll('.custom-field-row')
      rows.forEach(row => {
        const keyInput = row.querySelector('input[name="custom_field_keys[]"]')
        const valueInput = row.querySelector('input[name="custom_field_values[]"]')
        if (keyInput && valueInput && keyInput.value) {
          dataValues[keyInput.value] = valueInput.value
        }
      })
    }
    
    // Also include any data fields that might be in hidden inputs
    document.querySelectorAll('input[type="hidden"][name^="data["]').forEach(input => {
      const matches = input.name.match(/data\[(.*?)\]/)
      if (matches && matches.length > 1) {
        dataValues[matches[1]] = input.value
      }
    })
    
    return dataValues
  }
  
  convertToCustomFields(dataValues) {
    if (!this.hasCustomFieldsTarget) return
    
    // Clear existing custom fields
    this.customFieldsTarget.innerHTML = ''
    
    // Add custom fields for each data value
    Object.entries(dataValues).forEach(([key, value]) => {
      if (key && value) {
        this.addCustomField(key, value)
      }
    })
    
    // Add at least one empty custom field if there are none
    if (this.customFieldsTarget.querySelectorAll('.custom-field-row').length === 0) {
      this.addCustomField()
    }
  }
  
  generateRequiredFieldHtml(field, title, description, placeholder, value, required) {
    return `
      <div class="mb-4">
        <label for="data_${field}" class="block text-sm font-medium text-gray-700 dark:text-gray-300">
          ${title}${required ? ' <span class="text-red-500">*</span>' : ''}
        </label>
        ${description ? `<p class="mt-1 text-xs text-gray-500 dark:text-gray-400">${description}</p>` : ''}
        <input 
          type="text" 
          id="data_${field}"
          name="data[${field}]" 
          value="${this.escapeHtml(value)}"
          placeholder="${this.escapeHtml(placeholder)}"
          ${required ? 'required' : ''}
          class="mt-1 block w-full rounded-md border-gray-300 shadow-sm focus:border-blue-500 focus:ring-blue-500 dark:bg-gray-700 dark:border-gray-600 dark:text-white">
      </div>
    `
  }
  
  addCustomField(key = '', value = '') {
    if (!this.hasCustomFieldsTarget || !this.hasCustomTemplateTarget) return
    
    const template = this.customTemplateTarget.innerHTML
      .replace(/KEY_PLACEHOLDER/g, this.escapeHtml(key))
      .replace(/VALUE_PLACEHOLDER/g, this.escapeHtml(value))
    
    this.customFieldsTarget.insertAdjacentHTML('beforeend', template)
  }
  
  removeCustomField(event) {
    const row = event.target.closest('.custom-field-row')
    if (row) {
      row.remove()
    }
  }
  
  // Helper to convert snake_case to Title Case
  humanize(str) {
    return str
      .replace(/_/g, ' ')
      .replace(/\b\w/g, l => l.toUpperCase())
  }
  
  // Helper to escape HTML special characters
  escapeHtml(str) {
    if (!str) return ''
    
    return String(str)
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;')
  }
}
